# coding: utf-8

module UzuUzu
  #
  #
  #
  class Application
    #
    attr_accessor :name
    #
    attr_accessor :config_file
    #
    attr_accessor :environments
    #
    attr_accessor :controllers
    #
    attr_accessor :before_filters
    #
    attr_accessor :after_filters
    #
    attr_accessor :helpers
    #
    attr_accessor :controller_root
    #
    attr_accessor :helper_root
    #
    attr_accessor :view_root
    
    #
    #
    #
    def self.current
      ::Thread.current[:application]
    end
        
    #
    #
    #
    def initialize(options={})
      ::Thread.current[:application] = self
      options.each do |key, value|
        case(key)
        when :name, 'name'
          @name = value
        when :config, 'config'
          @config_file = value
        end
      end if options.kind_of?(Hash)
      @name ||= :uzuuzu
      @environments = Environments.new(@config_file)
      ::UzuUzu.apps[@name] = self
      yield self if block_given?
    end # initialize

    def self.controllers
      @@controllers ||= [Controller]
    end

    def controllers
      @controllers ||= self.class.controllers.clone
    end

    def self.before_filters
      @@before_filters ||= [Controller.method(:before_all)]
    end

    def before_filters
      @before_filters ||= self.class.before_filters.clone
    end

    def self.after_filters
      @@after_filters ||= [Controller.method(:after_all)]
    end
    
    def after_filters
      @after_filters ||= self.class.after_filters.clone
    end

    def self.helpers
      @@helpers ||= [
                     Helper::Controller,
                     Helper::Form,
                     Helper::Jquery,
                     Helper::Localize,
                     Helper::Renderer,
                     Helper::Route,
                     Helper::XHtml
                   ]
    end

    def helpers
      @helpers ||= self.class.helpers.clone
    end
    
    def self.controller_root
      @@controller_root ||= 'controller'
    end

    def controller_root
      @controller_root ||= self.class.controller_root.clone
    end

    def self.view_root
      @@view_root ||= 'view'
    end

    def view_root
      @view_root ||= self.class.view_root.clone
    end

    def self.helper_root
      @@helper_root ||= 'helper'
    end

    def helper_root
      @helper_root ||= self.class.helper_root.clone
    end

    #
    #
    #
    def call(env)
      Thread.current[:application] = self
      request = UzuUzu::Request.new(env)
      response = UzuUzu::Response.new
      auto_require(env)
      classies = find_controller(env)
      if classies.blank?
        di_thread(request, response, nil, nil,  nil, nil, nil)
        return not_found(response)
      end

      is_send = false
      action_return = nil
      render = nil
      classies.reverse.each do |map|
        controller_class = map[:controller]
        helper_module = map[:helper]
        query = map[:query]
        route = map[:route]
        controller = controller_class.new
        action, query = find_action(controller, query)
        di_thread(request, response, nil, nil, nil, query, route)
        next if action.blank?
        controller.extend(::UzuUzu::Controller)
        helper = Helper::Helper.new
        helpers.each do |h|
          helper.extend(h)
        end
        helper.extend(helper_module) if helper_module.kind_of?(::Module)
        di_thread(request, response, controller, helper,  action, query, route)
        is_send = true
        action_return = send_action(request, response, controller, action, query)
        if response.body.blank?
          render = helper.render(action)
        end
        break
      end
      
      unless is_send
        return not_found(response)
      end
      if response.status == 200 and response.body.blank?
        response.write render || action_return
      end
      response.finish
    end # call

    def require_noerror(path)
      require path
    rescue LoadError => e
    end
    #
    #
    #
    def auto_require(env)
      require_base = "/#{@name.to_s}"
      require_base = '' if @name == :uzuuzu || @name == '' || @name.blank?
      array = []
      if env['PATH_INFO'] == '/'
        require_noerror "#{controller_root}#{require_base}/index"
        require_noerror "#{helper_root}#{require_base}/index"
      end
      env['PATH_INFO'].split('/').each do |path|
        array << path
        begin
          require_noerror "#{controller_root}#{require_base}#{array.join('/')}/index"
          require_noerror "#{helper_root}#{require_base}#{array.join('/')}/index"
          require_noerror "#{controller_root}#{require_base}#{array.join('/')}"
          require_noerror "#{helper_root}#{require_base}#{array.join('/')}"
        rescue
        end
      end
    end # auto_require
    
    def get_class_map(name, controller, route, query)
      _c = controller.const_get(name) if controller.const_defined?(name)
      return nil if _c.nil?
      return nil if _c.name.index(controller.name).nil?
      _h = find_helper(name, route)
      map = {
        :controller => _c,
        :helper => _h,
        :route => route,
        :query => query
      }
    rescue NameError => e
    end
    
    #
    #
    #
    def find_helper(name, route)
      routes = route.split('/').map { |a| a.camel_case }
      if routes.blank? || name != routes[-1]
        routes << "#{name.camel_case}"
      end
      routes.shift if routes[0].blank?
      _base = "::#{@name.to_s.camel_case}"
      _base = '' if @name == :uzuuzu || @name == 'uzuuzu'
      helper = eval("#{_base}::#{helper_root.camel_case}")
      routes.each do |const_name|
        helper = helper.const_get(const_name) if helper.const_defined?(const_name)
      end
      helper
    rescue LoadError => e
    rescue NameError => e
    rescue => e
    end # find_helper
    
    #
    #
    #
    def find_controller(env)
      classies = []
      request_paths = env['PATH_INFO'].split('/')
      request_paths.shift if request_paths[0].blank?
      request_paths << ''
      
      _controllers = controllers.clone
      _base = "::#{@name.to_s.camel_case}"
      _base = '' if @name == :uzuuzu || @name == 'uzuuzu'
      begin
        _controllers << eval("#{_base}::#{controller_root.camel_case}")
      rescue LoadError => e
      rescue => e
      end
      _controllers.each do |c|
        _c = c
        _h = nil
        request_paths.each_with_index do |request_path, index|
          break if _c.nil?
          c_str = request_path.camel_case
          if c_str != 'Index'
            route = "#{request_paths[0...index].join('/')}"
            if route.blank?
              route = "#{env['SCRIPT_NAME']}/index"
              route.blank?
            else
              route = "#{env['SCRIPT_NAME']}/#{route}/index"
            end
            query = request_paths[index...-1]
            map = get_class_map('Index', _c, route, query)
            if map && map[:controller].kind_of?(::Class)
              classies << map
            end
          end
          next if c_str.blank?
          route = "#{env['SCRIPT_NAME']}/#{request_paths[0..index].join('/')}"
          query = request_paths[(index+1)...-1]
          map = get_class_map(c_str, _c, route, query)
          _c = nil
          if map
            classies << map if map[:controller].kind_of?(::Class)
            if map[:controller].kind_of?(::Class) || map[:controller].kind_of?(::Module)
              _c = map[:controller]
            end
          end
        end
      end
      classies
    end # find_controller
    
    #
    #
    #
    def di_thread(request, response, controller, helper, action, query, route)
      current = ::Thread.current
      current[:request] = request
      current[:response] = response
      current[:controller] = controller
      current[:helper] = helper
      current[:action] = action
      current[:query] = query
      current[:route] = route
    end # di_thread

    #
    #
    #
    def find_action(controller, query)
      action = nil
      index_flag = false
      query[-1] = query[-1].sub(/\.\w+$/, '') if query && query[-1]
      if query[0] && controller.respond_to?(query[0])
        action = query.shift
      end
      if action.nil? && controller.respond_to?(:index)
        action = 'index'
      end
      
      unless action.nil?
        arity = controller.method(action).arity
        owner = controller.method(action).owner
        unless controller.method(action).owner == controller.class
          index_flag = true
          if controller.respond_to?(:use_actions)
            controller.actions.each do |use_action|
              if use_action.to_s == action.to_s
                index_flag = false
              end
            end
          end
        end
        unless (arity >= 0 && query.size == arity) ||
               (arity < 0 && query.size >= ((-1 * arity)-1))
          index_flag = true
        end
        if index_flag && !(action == 'index') && controller.respond_to?(:index)
          query.unshift(action)
          action = 'index'
        end
        if controller.respond_to?(action)
          arity = controller.method(action).arity
          unless (arity >= 0 && query.size == arity) ||
                 (arity < 0 && query.size >= ((-1 * arity)-1))
            action = nil
          end
        else
          action = nil
        end
      end
      [action, query]
    end # find_action

    #
    #
    #
    def send_action(request, response, controller, action, query)
      value = nil
      catch(:finish) do
        before_filters.each do |filter|
          filter.call if filter.respond_to?(:call)
        end
        controller.send(:before) if controller.respond_to?(:before)
        controller.send("before_#{action}") if controller.respond_to?("before_#{action}")
        value = controller.send(action, *query)
      end
      catch(:finish) do
        controller.send("after_#{action}") if controller.respond_to?("after_#{action}")
      end
      catch(:finish) do
        controller.send(:after) if controller.respond_to?(:after)
      end
      after_filters.reverse.each do |filter|
        catch(:finish) do
          filter.call if filter.respond_to?(:call)
        end
      end
      value
    end # send_action

    def not_found(response)
      catch(:finish) do
        response.not_found
      end
      response.finish
    end
  end # Application
end # UzuUzu