# coding: utf-8

module UzuUzu
  #
  #
  #
  class Environments

    #
    DEFAULT_YAML = 'uzuuzu.yaml'
    #
    attr_accessor :map
    #
    attr_accessor :service_name
    #
    attr_accessor :dm
    #
    attr_accessor :s3
    #
    attr_accessor :logger
    #
    attr_accessor :memcache
    #
    attr_accessor :rack_session
    #
    attr_accessor :config
    #
    attr_accessor :lang
    #
    attr_accessor :localize
    
    def self.current
      ::UzuUzu.current.environments
    end
    
    def self.default_yaml=(yaml)
      @@default_yaml = yaml
    end

    def self.default_yaml
      @@default_yaml ||= DEFAULT_YAML
    end

    def initialize(yaml=nil)
      load_yaml(yaml||self.class.default_yaml)
      dm
    end

    def load_yaml(yaml)
      return if yaml.blank?
      yaml = ::YAML.load_file(yaml) unless yaml.kind_of?(Hash)
      setup(yaml)
    end

    def setup(map)
      return if map.blank?
      @config = map
      @lang = map['lang'] || 'ja'
      @service_name = map['service']
      load_localize
    end

    def load_localize
      @localize = {}
      if ::File.file?("./lang/#{@lang}.yaml")
        ::File.open("./lang/#{@lang}.yaml", :encoding => "UTF-8") do |f|
          @localize.merge!(::YAML.load(f))
        end
      end
      if ::File.file?(__DIR__("lang/#{@lang}.yaml"))
        ::File.open(__DIR__("lang/#{@lang}.yaml"), :encoding => "UTF-8") do |f|
          @localize.merge!(::YAML.load(f))
        end
      end
    end
    
    def map
      @map = {} if @map.nil?
      @map
    end

    def config
      return {} if @config.nil?
      @config || {}
    end
    
    def logger
      if @logger.nil?
        env = @config['logger'] unless @config.nil?
        @logger = Wrapper::Logger.new(env || {}).logger
      end
      @logger
    end

    def dm
      if @dm.nil?
        env = @config['dm'] unless @config.nil?
        @dm = Wrapper::Dm.new(env || {})
      end
      @dm
    end

    def s3
      if @s3.nil?
        env = @config['s3'] unless @config.nil?
        @s3 = Wrapper::S3.new(env || {})
      end
      @s3
    end

    def memcache
      if @memcache.nil?
        env = @config['memcache'] unless @config.nil?
        @memcache = Wrapper::Wrapper.new('memcache', env || {}).wrapper
        @memcache ||= {}
      end
      @memcache
    end
    
    def rack_session
      if @rack_session.nil?
        env = @config['rack_session'] unless @config.nil?
        @rack_session = Wrapper::Wrapper.new('rack_session', env || {}).wrapper
      end
      @rack_session
    end # rack_session
  end # Environments
end # UzuUzu