# coding: utf-8
require 'rack/session/abstract/id'

module UzuUzu
  module RackSession
    class Memcache < ::Rack::Session::Abstract::ID
      #
      attr_reader :adapter
      
      #
      #
      #
      def initialize(env=nil)
        @adapter = :memcache
        super(nil, env)
        @mutex = Mutex.new
        @pool = ::UzuUzu::Wrapper::Wrapper.new('memcache', env['config']).wrapper
      end
      
      def app(app)
        @app = app
      end
      
      def generate_sid
        loop do
          sid = super
          break sid unless @pool.get(sid)
        end
      end

      def get_session(env, sid)
        with_lock(env, [nil, {}]) do
          unless sid and session = @pool.get(sid)
            sid, session = generate_sid, {}
            unless /^STORED/ =~ @pool.add(sid, session)
              raise "Session collision on '#{sid.inspect}'"
            end
          end
          [sid, session]
        end
      end

      def set_session(env, session_id, new_session, options)
        with_lock(env, false) do
          @pool.set session_id, new_session
          session_id
        end
      end

      def destroy_session(env, session_id, options)
        with_lock(env) do
          @pool.delete(session_id)
          generate_sid unless options[:drop]
        end
      end

      def with_lock(env, default=nil)
        @mutex.lock if env['rack.multithread']
        yield
      rescue
        default
      ensure
        @mutex.unlock if @mutex.locked?
      end
    end # Leveldb
  end # Kvs
end # UzuUzu