# coding: utf-8
require 'uri'
require 'net/http'
require 'net/https'

module UzuUzu
  #
  #
  #
  class Response < Rack::Response
    include ::UzuUzu::Helper::Controller
    
    #
    #
    #
    def self.current
      Thread.current[:response]
    end
    
    #
    #
    #
    def rack_mime_type(type, value=nil)
      return type if type.nil? || type.to_s.include?('/')
      type = ".#{type}" unless type.to_s[0] == ?.
      return Rack::Mime.mime_type(type, nil) if value.nil?
      Rack::Mime::MIME_TYPES[type] = value
    end
    
    #
    #
    #
    def mime_type(mime)
      @header["Content-Type"] = rack_mime_type(mime) || mime if mime
    end
    
    #
    #
    #
    def respond(string, mime=nil, status=200)
      @status = 200
      mime_type(mime)
      write string
      throw :finish
    end
    
    #
    #
    #
    def respond_to
      return unless block_given?
      @status = 200
      contents = yield(request.wish)
      respond(contents, request.wish) if contents and successful?
    end
      
    def send_file()
      
    end
    
    def send_binary()
      
    end
    
    #
    #
    #
    def server_error(code=500, options={})
      redirect "/error/#{code}"
    end # error
    
    #
    #
    #
    def not_found(code=404, options={})
      server_error(code, options)
    end
    
    #
    #
    #
    def redirect(uri, options={})
      if not uri =~ /^https?:\/\//
        abs_uri = "#{request.scheme}://#{request.host}"

        if request.scheme == 'https' && request.port != 443 ||
        request.scheme == 'http' && request.port != 80
          abs_uri << ":#{request.port}"
        end
        uri = (abs_uri << uri)
      end
      uri << query_string(options)
      super(uri)
      throw :finish
    end # redirect
    
    #
    #
    #
    def reverse_proxy(url, opts={})
      uri = URI.parse(url)
      headers = opts['headers'] if opts['headers']
      headers ||= opts[:headers] if opts[:headers]
      headers ||= Rack::Utils::HeaderHash.new
      request.env.each do |key, value|
        if key =~ /HTTP_(.*)/
          headers[$1] = value
        end
      end
      headers['HOST'] = uri.host unless (opts[:host] || opts['host'])
      Net::HTTP.version_1_2
      http = Net::HTTP.new( uri.host, uri.port )
      http.read_timeout = opts[:timeout] if opts[:timeout]
      http.read_timeout = opts['timeout'] if opts['timeout']
      if uri.scheme == 'https' && (opts[:ssl] || opts['ssl'])
        http.verify_mode = OpenSSL::SSL::VERIFY_PEER
      else
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
      http.start do |http|
        m = request.request_method
        case m
        when 'GET', 'HEAD', 'DELETE', 'OPTIONS', 'TRACE'
          req = Net::HTTP.const_get(m.capitalize).new("#{uri.path}?#{uri.query}", headers)
          req.basic_auth opts[:username], opts[:password] if opts[:username] and opts[:password]
          req.basic_auth opts['username'], opts['password'] if opts['username'] and opts['password']
        when 'PUT', 'POST'
          req = Net::HTTP.const_get(m.capitalize).new("#{uri.path}?#{uri.query}", headers)
          req.basic_auth opts[:username], opts[:password] if opts[:username] and opts[:password]
          req.basic_auth opts['username'], opts['password'] if opts['username'] and opts['password']

          if request.body.respond_to?(:read) && request.body.respond_to?(:rewind)
            body = request.body.read
            req.content_length = body.size
            request.body.rewind
          else
            req.content_length = request.body.size
          end

          req.content_type = request.content_type unless request.content_type.nil?
          req.body_stream = request.body
        else
          raise "method not supported: #{m}"
        end # case m
        
        body = ''
        res = http.request(req) do |res|
          res.read_body do |segment|
            body << segment
          end
        end
        response_headers = Rack::Utils::HeaderHash.new(res.to_hash)
        response_headers.delete('status')
        response_headers.delete('transfer-encoding')
        response_headers.each do |key, value|
            response.header[key] = value
        end
        response.status = 200
        response.body = [body]
      end # http
      
      throw :finish
    end
    #
    #
    #
    def last_modified(time)
      return unless time
      if time.respond_to?(:to_time)
          time = time.to_time
      else
          time = ::Time.parse(time.to_s)
      end
      @header['Last-Modified'] = time.httpdate
      if ::Time.httpdate(request.env['HTTP_IF_MODIFIED_SINCE']).to_i >= time.to_i
        @status = 304
        throw :finish
      end
    end # last_modified
    
    #
    #
    #
    def etag(value, kind=:strong)
      raise ::TypeError, ":strong or :weak expected" if ![:strong,:weak].include?(kind)
      value = '"%s"' % value
      value = 'W/' + value if kind == :weak
      @header['ETag'] = value

      # Conditional GET check
      if etags = request.env['HTTP_IF_NONE_MATCH']
        etags = etags.split(/\s*,\s*/)
        if etags.include?(value) || etags.include?('*')
          @status = 304
          throw :finish
        end
      end
    end # etag
    
  end # Response
end # UzuUzu