# coding: utf-8
require 'dm-core'

module UzuUzu
  module Wrapper
    class Dm
      #
      attr_reader :adapter
      #
      attr_reader :repositories
      
      #
      #
      #
      def initialize(env=nil)
        if env.nil?
        return
        end
        @env = env.clone

        unless @env['logger'].nil?
          DataMapper::logger = ::UzuUzu::Wrapper::Logger.new(@env['logger']).logger
        end
        @repositories = []
        @env.each do |key, value|
          @repositories.push key.to_sym
          if value['adapter']
            @adapter = value['adapter']
            DataMapper.setup(key.to_sym, value)
            if value['logger']
              case(value['adapter'])
              when 'sqlite3'
                DataObjects::Sqlite3.logger = ::UzuUzu::Wrapper::Logger.new(value['logger']).logger
              when 'postgres'
                DataObjects::Postgres.logger = ::UzuUzu::Wrapper::Logger.new(value['logger']).logger
              when 'mysql'
                DataObjects::MySql.logger = ::UzuUzu::Wrapper::Logger.new(value['logger']).logger
              end
            end
          end
        end # env.each
      end # new

      #
      #
      #
      def import_file(model, file, overwrite=false)
        unless ::File.file?(file)
          raise "UzuUzu fixture import error file not found : #{file}"
        end
        ext = ::File.extname(file)
        case(ext)
        when '.yml', '.yaml'
          import_yaml(model, ::YAML.load_file(file), overwrite)
        when '.xml'
          # TODO
        when '.json', '.js'
          # TODO
        when '.csv'
          begin
            require 'fastercsv'
            ::FasterCSV.foreach(file) do |row|
              import_csv(model, row, overwrite)
            end
          rescue
            require "csv"
            ::CSV.open(file) do |row|
              import_csv(model, row, overwrite)
            end
          end
        end
        nil
      end # import_file
      
      def import_yaml(model, yaml, overwrite=false)
        ::UzuUzu.logger.debug 'import yaml'
        if yaml.kind_of?(Array)
          yaml.each do |map|
            ::UzuUzu.logger.debug "import #{map.inspect}"
            resource = model.first(:id => map[:id]) unless map[:id].nil?
            if resource.nil?
              resource = model.new(map)
              unless resource.valid?
                ::UzuUzu.logger.error resource.errors
              end
              resource.save
            elsif overwrite
              resource.update(map)
              unless resource.valid?
                ::UzuUzu.logger.error resource.errors
              end
            end
          end
        elsif yaml.kind_of?(Hash)
          ::UzuUzu.logger.debug "import #{yaml.inspect}"
          resource = model.first(:id => yaml[:id]) unless yaml[:id].nil?
          if resource.nil?
            resource = model.new(yaml)
            unless resource.valid?
              ::UzuUzu.logger.error resource.errors
            end
            resource.save
          elsif overwrite
            resource.update(yaml)
            unless resource.valid?
              ::UzuUzu.logger.error resource.errors
            end
          end
        else
          ::UzuUzu.logger.error "import type miss match #{yaml.inspect}"
        end
        ::UzuUzu.logger.debug 'imported yaml'
      end # import_yaml
      
      def import_json(model, json, overwrite=false)
        # TODO
      end # import_json
      
      def import_xml(model, xml, overwrite=false)
        # TODO
      end # import_xml
      
      def import_csv(model, csv, overwrite=false)
        resource = model.first(:id => csv[0])
        csv.each_with_index do |culumn, index|
          if resource.nil? || overwrite
            resource ||= model.new
            resource.properties.each_with_index do |property, column_no|
              resource[property.name] = culumn if index = column_no
            end
            resource.save
          end
        end
      end # import_csv
      
    end # Dm
  end # Wrapper
end # UzuUzu